@@ -1,5 +1,5 @@
 /*
- * Copyright 2014-2017 MongoDB, Inc.
+ * Copyright 2014-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,21 +14,15 @@
  * limitations under the License.
  */
 
+#include "bson/bson.h"
+
 #include <php.h>
-#include <Zend/zend_interfaces.h>
-#include <ext/standard/php_var.h>
-#if PHP_VERSION_ID >= 70000
 #include <zend_smart_str.h>
-#else
-#include <ext/standard/php_smart_str.h>
-#endif
+#include <ext/standard/php_var.h>
+#include <Zend/zend_interfaces.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "phongo_compat.h"
 #include "php_phongo.h"
+#include "phongo_error.h"
 
 #define PHONGO_OID_SIZE sizeof(((php_phongo_objectid_t*) 0)->oid)
 #define PHONGO_OID_LEN (PHONGO_OID_SIZE - 1)
@@ -51,7 +45,7 @@ static bool php_phongo_objectid_init(php_phongo_objectid_t* intern)
 
 /* Initialize the object from a hex string and return whether it was successful.
  * An exception will be thrown on error. */
-static bool php_phongo_objectid_init_from_hex_string(php_phongo_objectid_t* intern, const char* hex, phongo_zpp_char_len hex_len TSRMLS_DC) /* {{{ */
+static bool php_phongo_objectid_init_from_hex_string(php_phongo_objectid_t* intern, const char* hex, size_t hex_len) /* {{{ */
 {
 	if (bson_oid_is_valid(hex, hex_len)) {
 		bson_oid_t oid;
@@ -63,55 +57,70 @@ static bool php_phongo_objectid_init_from_hex_string(php_phongo_objectid_t* inte
 		return true;
 	}
 
-	phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Error parsing ObjectId string: %s", hex);
+	phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Error parsing ObjectId string: %s", hex);
 
 	return false;
 } /* }}} */
 
 /* Initialize the object from a HashTable and return whether it was successful.
  * An exception will be thrown on error. */
-static bool php_phongo_objectid_init_from_hash(php_phongo_objectid_t* intern, HashTable* props TSRMLS_DC) /* {{{ */
+static bool php_phongo_objectid_init_from_hash(php_phongo_objectid_t* intern, HashTable* props) /* {{{ */
 {
-#if PHP_VERSION_ID >= 70000
 	zval* z_oid;
 
 	z_oid = zend_hash_str_find(props, "oid", sizeof("oid") - 1);
 
 	if (z_oid && Z_TYPE_P(z_oid) == IS_STRING) {
-		return php_phongo_objectid_init_from_hex_string(intern, Z_STRVAL_P(z_oid), Z_STRLEN_P(z_oid) TSRMLS_CC);
+		return php_phongo_objectid_init_from_hex_string(intern, Z_STRVAL_P(z_oid), Z_STRLEN_P(z_oid));
 	}
-#else
-	zval** z_oid;
 
-	if (zend_hash_find(props, "oid", sizeof("oid"), (void**) &z_oid) == SUCCESS && Z_TYPE_PP(z_oid) == IS_STRING) {
-		return php_phongo_objectid_init_from_hex_string(intern, Z_STRVAL_PP(z_oid), Z_STRLEN_PP(z_oid) TSRMLS_CC);
-	}
-#endif
-
-	phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "%s initialization requires \"oid\" string field", ZSTR_VAL(php_phongo_objectid_ce->name));
+	phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "%s initialization requires \"oid\" string field", ZSTR_VAL(php_phongo_objectid_ce->name));
 	return false;
 } /* }}} */
 
+static HashTable* php_phongo_objectid_get_properties_hash(phongo_compat_object_handler_type* object, bool is_temp) /* {{{ */
+{
+	php_phongo_objectid_t* intern;
+	HashTable*             props;
+
+	intern = Z_OBJ_OBJECTID(PHONGO_COMPAT_GET_OBJ(object));
+
+	PHONGO_GET_PROPERTY_HASH_INIT_PROPS(is_temp, intern, props, 1);
+
+	if (!intern->initialized) {
+		return props;
+	}
+
+	{
+		zval zv;
+
+		ZVAL_STRING(&zv, intern->oid);
+		zend_hash_str_update(props, "oid", sizeof("oid") - 1, &zv);
+	}
+
+	return props;
+} /* }}} */
+
 /* {{{ proto void MongoDB\BSON\ObjectId::__construct([string $id])
    Constructs a new BSON ObjectId type, optionally from a hex string. */
 static PHP_METHOD(ObjectId, __construct)
 {
-	php_phongo_objectid_t* intern;
 	zend_error_handling    error_handling;
+	php_phongo_objectid_t* intern;
 	char*                  id = NULL;
-	phongo_zpp_char_len    id_len;
+	size_t                 id_len;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling TSRMLS_CC);
 	intern = Z_OBJECTID_OBJ_P(getThis());
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|s!", &id, &id_len) == FAILURE) {
-		zend_restore_error_handling(&error_handling TSRMLS_CC);
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "|s!", &id, &id_len) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
-	zend_restore_error_handling(&error_handling TSRMLS_CC);
+	zend_restore_error_handling(&error_handling);
 
 	if (id) {
-		php_phongo_objectid_init_from_hex_string(intern, id, id_len TSRMLS_CC);
+		php_phongo_objectid_init_from_hex_string(intern, id, id_len);
 	} else {
 		php_phongo_objectid_init(intern);
 	}
@@ -121,63 +130,79 @@ static PHP_METHOD(ObjectId, __construct)
 */
 static PHP_METHOD(ObjectId, getTimestamp)
 {
+	zend_error_handling    error_handling;
 	php_phongo_objectid_t* intern;
 	bson_oid_t             tmp_oid;
 
 	intern = Z_OBJECTID_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	bson_oid_init_from_string(&tmp_oid, intern->oid);
 	RETVAL_LONG(bson_oid_get_time_t(&tmp_oid));
 } /* }}} */
 
-/* {{{ proto MongoDB\BSON\ObjectId::__set_state(array $properties)
+/* {{{ proto MongoDB\BSON\ObjectId MongoDB\BSON\ObjectId::__set_state(array $properties)
 */
 static PHP_METHOD(ObjectId, __set_state)
 {
+	zend_error_handling    error_handling;
 	php_phongo_objectid_t* intern;
 	HashTable*             props;
 	zval*                  array;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a", &array) == FAILURE) {
-		RETURN_FALSE;
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a", &array) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
+		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	object_init_ex(return_value, php_phongo_objectid_ce);
 
 	intern = Z_OBJECTID_OBJ_P(return_value);
 	props  = Z_ARRVAL_P(array);
 
-	php_phongo_objectid_init_from_hash(intern, props TSRMLS_CC);
+	php_phongo_objectid_init_from_hash(intern, props);
 } /* }}} */
 
 /* {{{ proto string MongoDB\BSON\ObjectId::__toString()
 */
 static PHP_METHOD(ObjectId, __toString)
 {
+	zend_error_handling    error_handling;
 	php_phongo_objectid_t* intern;
 
 	intern = Z_OBJECTID_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
-	PHONGO_RETURN_STRINGL(intern->oid, PHONGO_OID_LEN);
+	RETURN_STRINGL(intern->oid, PHONGO_OID_LEN);
 } /* }}} */
 
 /* {{{ proto array MongoDB\BSON\ObjectId::jsonSerialize()
 */
 static PHP_METHOD(ObjectId, jsonSerialize)
 {
+	zend_error_handling    error_handling;
 	php_phongo_objectid_t* intern;
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	intern = Z_OBJECTID_OBJ_P(getThis());
 
@@ -189,28 +214,26 @@ static PHP_METHOD(ObjectId, jsonSerialize)
 */
 static PHP_METHOD(ObjectId, serialize)
 {
+	zend_error_handling    error_handling;
 	php_phongo_objectid_t* intern;
-	ZVAL_RETVAL_TYPE       retval;
+	zval                   retval;
 	php_serialize_data_t   var_hash;
 	smart_str              buf = { 0 };
 
 	intern = Z_OBJECTID_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
-#if PHP_VERSION_ID >= 70000
 	array_init_size(&retval, 1);
 	ADD_ASSOC_STRINGL(&retval, "oid", intern->oid, PHONGO_OID_LEN);
-#else
-	ALLOC_INIT_ZVAL(retval);
-	array_init_size(retval, 1);
-	ADD_ASSOC_STRINGL(retval, "oid", intern->oid, PHONGO_OID_LEN);
-#endif
 
 	PHP_VAR_SERIALIZE_INIT(var_hash);
-	php_var_serialize(&buf, &retval, &var_hash TSRMLS_CC);
+	php_var_serialize(&buf, &retval, &var_hash);
 	smart_str_0(&buf);
 	PHP_VAR_SERIALIZE_DESTROY(var_hash);
 
@@ -224,49 +247,60 @@ static PHP_METHOD(ObjectId, serialize)
 */
 static PHP_METHOD(ObjectId, unserialize)
 {
-	php_phongo_objectid_t* intern;
 	zend_error_handling    error_handling;
+	php_phongo_objectid_t* intern;
 	char*                  serialized;
-	phongo_zpp_char_len    serialized_len;
-#if PHP_VERSION_ID >= 70000
-	zval props;
-#else
-	zval* props;
-#endif
+	size_t                 serialized_len;
+	zval                   props;
 	php_unserialize_data_t var_hash;
 
 	intern = Z_OBJECTID_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling TSRMLS_CC);
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &serialized, &serialized_len) == FAILURE) {
-		zend_restore_error_handling(&error_handling TSRMLS_CC);
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &serialized, &serialized_len) == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
-	zend_restore_error_handling(&error_handling TSRMLS_CC);
+	zend_restore_error_handling(&error_handling);
 
-#if PHP_VERSION_ID < 70000
-	ALLOC_INIT_ZVAL(props);
-#endif
 	PHP_VAR_UNSERIALIZE_INIT(var_hash);
-	if (!php_var_unserialize(&props, (const unsigned char**) &serialized, (unsigned char*) serialized + serialized_len, &var_hash TSRMLS_CC)) {
+	if (!php_var_unserialize(&props, (const unsigned char**) &serialized, (unsigned char*) serialized + serialized_len, &var_hash)) {
 		zval_ptr_dtor(&props);
-		phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "%s unserialization failed", ZSTR_VAL(php_phongo_objectid_ce->name));
+		phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "%s unserialization failed", ZSTR_VAL(php_phongo_objectid_ce->name));
 
 		PHP_VAR_UNSERIALIZE_DESTROY(var_hash);
 		return;
 	}
 	PHP_VAR_UNSERIALIZE_DESTROY(var_hash);
 
-#if PHP_VERSION_ID >= 70000
-	php_phongo_objectid_init_from_hash(intern, HASH_OF(&props) TSRMLS_CC);
-#else
-	php_phongo_objectid_init_from_hash(intern, HASH_OF(props) TSRMLS_CC);
-#endif
+	php_phongo_objectid_init_from_hash(intern, HASH_OF(&props));
 	zval_ptr_dtor(&props);
 } /* }}} */
 
+/* {{{ proto array MongoDB\Driver\ObjectId::__serialize()
+*/
+static PHP_METHOD(ObjectId, __serialize)
+{
+	PHONGO_PARSE_PARAMETERS_NONE();
+
+	RETURN_ARR(php_phongo_objectid_get_properties_hash(PHONGO_COMPAT_OBJ_P(getThis()), true));
+} /* }}} */
+
+/* {{{ proto void MongoDB\Driver\ObjectId::__unserialize(array $data)
+*/
+static PHP_METHOD(ObjectId, __unserialize)
+{
+	zval* data;
+
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_ARRAY(data)
+	PHONGO_PARSE_PARAMETERS_END();
+
+	php_phongo_objectid_init_from_hash(Z_OBJECTID_OBJ_P(getThis()), Z_ARRVAL_P(data));
+} /* }}} */
+
 /* {{{ MongoDB\BSON\ObjectId function entries */
+/* clang-format off */
 ZEND_BEGIN_ARG_INFO_EX(ai_ObjectId___construct, 0, 0, 0)
 	ZEND_ARG_INFO(0, id)
 ZEND_END_ARG_INFO()
@@ -275,6 +309,16 @@ ZEND_BEGIN_ARG_INFO_EX(ai_ObjectId___set_state, 0, 0, 1)
 	ZEND_ARG_ARRAY_INFO(0, properties, 0)
 ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_WITH_RETURN_TYPE_INFO_EX(ai_ObjectId___toString, 0, 0, IS_STRING, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(ai_ObjectId___unserialize, 0, 0, 1)
+	ZEND_ARG_ARRAY_INFO(0, data, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(ai_ObjectId_jsonSerialize, 0, 0, IS_ARRAY, 0)
+ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO_EX(ai_ObjectId_unserialize, 0, 0, 1)
 	ZEND_ARG_INFO(0, serialized)
 ZEND_END_ARG_INFO()
@@ -283,82 +327,58 @@ ZEND_BEGIN_ARG_INFO_EX(ai_ObjectId_void, 0, 0, 0)
 ZEND_END_ARG_INFO()
 
 static zend_function_entry php_phongo_objectid_me[] = {
-	/* clang-format off */
 	PHP_ME(ObjectId, __construct, ai_ObjectId___construct, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(ObjectId, getTimestamp, ai_ObjectId_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(ObjectId, __serialize, ai_ObjectId_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(ObjectId, __set_state, ai_ObjectId___set_state, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-	PHP_ME(ObjectId, __toString, ai_ObjectId_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
-	PHP_ME(ObjectId, jsonSerialize, ai_ObjectId_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(ObjectId, __toString, ai_ObjectId___toString, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(ObjectId, __unserialize, ai_ObjectId___unserialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(ObjectId, jsonSerialize, ai_ObjectId_jsonSerialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(ObjectId, serialize, ai_ObjectId_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(ObjectId, unserialize, ai_ObjectId_unserialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_FE_END
-	/* clang-format on */
 };
+/* clang-format on */
 /* }}} */
 
 /* {{{ MongoDB\BSON\ObjectId object handlers */
 static zend_object_handlers php_phongo_handler_objectid;
 
-static void php_phongo_objectid_free_object(phongo_free_object_arg* object TSRMLS_DC) /* {{{ */
+static void php_phongo_objectid_free_object(zend_object* object) /* {{{ */
 {
 	php_phongo_objectid_t* intern = Z_OBJ_OBJECTID(object);
 
-	zend_object_std_dtor(&intern->std TSRMLS_CC);
+	zend_object_std_dtor(&intern->std);
 
 	if (intern->properties) {
 		zend_hash_destroy(intern->properties);
 		FREE_HASHTABLE(intern->properties);
 	}
-
-#if PHP_VERSION_ID < 70000
-	efree(intern);
-#endif
 } /* }}} */
 
-static phongo_create_object_retval php_phongo_objectid_create_object(zend_class_entry* class_type TSRMLS_DC) /* {{{ */
+static zend_object* php_phongo_objectid_create_object(zend_class_entry* class_type) /* {{{ */
 {
-	php_phongo_objectid_t* intern = NULL;
+	php_phongo_objectid_t* intern = zend_object_alloc(sizeof(php_phongo_objectid_t), class_type);
 
-	intern = PHONGO_ALLOC_OBJECT_T(php_phongo_objectid_t, class_type);
-
-	zend_object_std_init(&intern->std, class_type TSRMLS_CC);
+	zend_object_std_init(&intern->std, class_type);
 	object_properties_init(&intern->std, class_type);
 
-#if PHP_VERSION_ID >= 70000
 	intern->std.handlers = &php_phongo_handler_objectid;
 
 	return &intern->std;
-#else
-	{
-		zend_object_value retval;
-		retval.handle   = zend_objects_store_put(intern, (zend_objects_store_dtor_t) zend_objects_destroy_object, php_phongo_objectid_free_object, NULL TSRMLS_CC);
-		retval.handlers = &php_phongo_handler_objectid;
-
-		return retval;
-	}
-#endif
 } /* }}} */
 
-static phongo_create_object_retval php_phongo_objectid_clone_object(zval* object TSRMLS_DC) /* {{{ */
+static zend_object* php_phongo_objectid_clone_object(phongo_compat_object_handler_type* object) /* {{{ */
 {
-	php_phongo_objectid_t*      intern;
-	php_phongo_objectid_t*      new_intern;
-	phongo_create_object_retval new_object;
+	php_phongo_objectid_t* intern;
+	php_phongo_objectid_t* new_intern;
+	zend_object*           new_object;
 
-	intern     = Z_OBJECTID_OBJ_P(object);
-	new_object = php_phongo_objectid_create_object(Z_OBJCE_P(object) TSRMLS_CC);
+	intern     = Z_OBJ_OBJECTID(PHONGO_COMPAT_GET_OBJ(object));
+	new_object = php_phongo_objectid_create_object(PHONGO_COMPAT_GET_OBJ(object)->ce);
 
-#if PHP_VERSION_ID >= 70000
 	new_intern = Z_OBJ_OBJECTID(new_object);
-	zend_objects_clone_members(&new_intern->std, &intern->std TSRMLS_CC);
-#else
-	{
-		zend_object_handle handle = Z_OBJ_HANDLE_P(object);
-
-		new_intern = (php_phongo_objectid_t*) zend_object_store_get_object_by_handle(new_object.handle TSRMLS_CC);
-		zend_objects_clone_members(&new_intern->std, new_object, &intern->std, handle TSRMLS_CC);
-	}
-#endif
+	zend_objects_clone_members(&new_intern->std, &intern->std);
 
 	// Use memcpy to copy bson value to avoid converting to string and back
 	memcpy(&new_intern->oid, &intern->oid, PHONGO_OID_SIZE);
@@ -367,67 +387,28 @@ static phongo_create_object_retval php_phongo_objectid_clone_object(zval* object
 	return new_object;
 } /* }}} */
 
-static int php_phongo_objectid_compare_objects(zval* o1, zval* o2 TSRMLS_DC) /* {{{ */
+static int php_phongo_objectid_compare_objects(zval* o1, zval* o2) /* {{{ */
 {
 	php_phongo_objectid_t* intern1;
 	php_phongo_objectid_t* intern2;
 
+	ZEND_COMPARE_OBJECTS_FALLBACK(o1, o2);
+
 	intern1 = Z_OBJECTID_OBJ_P(o1);
 	intern2 = Z_OBJECTID_OBJ_P(o2);
 
 	return strcmp(intern1->oid, intern2->oid);
 } /* }}} */
 
-static HashTable* php_phongo_objectid_get_gc(zval* object, phongo_get_gc_table table, int* n TSRMLS_DC) /* {{{ */
-{
-	*table = NULL;
-	*n     = 0;
-
-	return Z_OBJECTID_OBJ_P(object)->properties;
-} /* }}} */
-
-static HashTable* php_phongo_objectid_get_properties_hash(zval* object, bool is_debug TSRMLS_DC) /* {{{ */
-{
-	php_phongo_objectid_t* intern;
-	HashTable*             props;
-
-	intern = Z_OBJECTID_OBJ_P(object);
-
-	PHONGO_GET_PROPERTY_HASH_INIT_PROPS(is_debug, intern, props, 1);
-
-	if (!intern->initialized) {
-		return props;
-	}
-
-#if PHP_VERSION_ID >= 70000
-	{
-		zval zv;
-
-		ZVAL_STRING(&zv, intern->oid);
-		zend_hash_str_update(props, "oid", sizeof("oid") - 1, &zv);
-	}
-#else
-	{
-		zval* zv;
-
-		MAKE_STD_ZVAL(zv);
-		ZVAL_STRING(zv, intern->oid, 1);
-		zend_hash_update(props, "oid", sizeof("oid"), &zv, sizeof(zv), NULL);
-	}
-#endif
-
-	return props;
-} /* }}} */
-
-static HashTable* php_phongo_objectid_get_debug_info(zval* object, int* is_temp TSRMLS_DC) /* {{{ */
+static HashTable* php_phongo_objectid_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
 {
 	*is_temp = 1;
-	return php_phongo_objectid_get_properties_hash(object, true TSRMLS_CC);
+	return php_phongo_objectid_get_properties_hash(object, true);
 } /* }}} */
 
-static HashTable* php_phongo_objectid_get_properties(zval* object TSRMLS_DC) /* {{{ */
+static HashTable* php_phongo_objectid_get_properties(phongo_compat_object_handler_type* object) /* {{{ */
 {
-	return php_phongo_objectid_get_properties_hash(object, false TSRMLS_CC);
+	return php_phongo_objectid_get_properties_hash(object, false);
 } /* }}} */
 /* }}} */
 
@@ -436,32 +417,36 @@ void php_phongo_objectid_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	zend_class_entry ce;
 
 	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\BSON", "ObjectId", php_phongo_objectid_me);
-	php_phongo_objectid_ce                = zend_register_internal_class(&ce TSRMLS_CC);
+	php_phongo_objectid_ce                = zend_register_internal_class(&ce);
 	php_phongo_objectid_ce->create_object = php_phongo_objectid_create_object;
 	PHONGO_CE_FINAL(php_phongo_objectid_ce);
 
-	zend_class_implements(php_phongo_objectid_ce TSRMLS_CC, 1, php_phongo_objectid_interface_ce);
-	zend_class_implements(php_phongo_objectid_ce TSRMLS_CC, 1, php_phongo_json_serializable_ce);
-	zend_class_implements(php_phongo_objectid_ce TSRMLS_CC, 1, php_phongo_type_ce);
-	zend_class_implements(php_phongo_objectid_ce TSRMLS_CC, 1, zend_ce_serializable);
+	zend_class_implements(php_phongo_objectid_ce, 1, php_phongo_objectid_interface_ce);
+	zend_class_implements(php_phongo_objectid_ce, 1, php_phongo_json_serializable_ce);
+	zend_class_implements(php_phongo_objectid_ce, 1, php_phongo_type_ce);
+	zend_class_implements(php_phongo_objectid_ce, 1, zend_ce_serializable);
+
+#if PHP_VERSION_ID >= 80000
+	zend_class_implements(php_phongo_objectid_ce, 1, zend_ce_stringable);
+#endif
 
 	memcpy(&php_phongo_handler_objectid, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
-	php_phongo_handler_objectid.clone_obj       = php_phongo_objectid_clone_object;
-	php_phongo_handler_objectid.compare_objects = php_phongo_objectid_compare_objects;
-	php_phongo_handler_objectid.get_debug_info  = php_phongo_objectid_get_debug_info;
-	php_phongo_handler_objectid.get_gc          = php_phongo_objectid_get_gc;
-	php_phongo_handler_objectid.get_properties  = php_phongo_objectid_get_properties;
-#if PHP_VERSION_ID >= 70000
-	php_phongo_handler_objectid.free_obj = php_phongo_objectid_free_object;
-	php_phongo_handler_objectid.offset   = XtOffsetOf(php_phongo_objectid_t, std);
-#endif
+	PHONGO_COMPAT_SET_COMPARE_OBJECTS_HANDLER(objectid);
+	php_phongo_handler_objectid.clone_obj      = php_phongo_objectid_clone_object;
+	php_phongo_handler_objectid.get_debug_info = php_phongo_objectid_get_debug_info;
+	php_phongo_handler_objectid.get_properties = php_phongo_objectid_get_properties;
+	php_phongo_handler_objectid.free_obj       = php_phongo_objectid_free_object;
+	php_phongo_handler_objectid.offset         = XtOffsetOf(php_phongo_objectid_t, std);
 } /* }}} */
 
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
+void phongo_objectid_init(zval* return_value, const bson_oid_t* oid) /* {{{ */
+{
+	php_phongo_objectid_t* intern;
+
+	object_init_ex(return_value, php_phongo_objectid_ce);
+
+	intern = Z_OBJECTID_OBJ_P(return_value);
+	bson_oid_to_string(oid, intern->oid);
+	intern->initialized = true;
+}
+/* }}} */
