@@ -15,16 +15,12 @@
  */
 
 #include <php.h>
-#include <Zend/zend_interfaces.h>
-#include <ext/standard/php_var.h>
 #include <zend_smart_str.h>
+#include <ext/standard/php_var.h>
+#include <Zend/zend_interfaces.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "phongo_compat.h"
 #include "php_phongo.h"
+#include "phongo_error.h"
 
 zend_class_entry* php_phongo_regex_ce;
 
@@ -396,9 +392,7 @@ static void php_phongo_regex_free_object(zend_object* object) /* {{{ */
 
 static zend_object* php_phongo_regex_create_object(zend_class_entry* class_type) /* {{{ */
 {
-	php_phongo_regex_t* intern = NULL;
-
-	intern = PHONGO_ALLOC_OBJECT_T(php_phongo_regex_t, class_type);
+	php_phongo_regex_t* intern = zend_object_alloc(sizeof(php_phongo_regex_t), class_type);
 
 	zend_object_std_init(&intern->std, class_type);
 	object_properties_init(&intern->std, class_type);
@@ -483,12 +477,3 @@ void php_phongo_regex_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	php_phongo_handler_regex.free_obj       = php_phongo_regex_free_object;
 	php_phongo_handler_regex.offset         = XtOffsetOf(php_phongo_regex_t, std);
 } /* }}} */
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
