@@ -1,5 +1,5 @@
 /*
- * Copyright 2014-2017 MongoDB, Inc.
+ * Copyright 2014-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,11 +17,6 @@
 #include <php.h>
 #include <zend_exceptions.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "phongo_compat.h"
 #include "php_phongo.h"
 
 zend_class_entry* php_phongo_exception_ce;
@@ -40,12 +35,3 @@ void php_phongo_exception_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	php_phongo_exception_ce = zend_register_internal_interface(&ce);
 	zend_class_implements(php_phongo_exception_ce, 1, zend_ce_throwable);
 } /* }}} */
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
