@@ -14,17 +14,19 @@
  * limitations under the License.
  */
 
+#include "bson/bson.h"
+#include "mongoc/mongoc.h"
+
 #include <php.h>
 #include <Zend/zend_interfaces.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "php_array_api.h"
-#include "phongo_compat.h"
+
 #include "php_phongo.h"
-#include "php_bson.h"
+#include "phongo_bson_encode.h"
+#include "phongo_error.h"
+
+#include "MongoDB/ReadConcern.h"
 
 zend_class_entry* php_phongo_query_ce;
 
@@ -411,9 +413,7 @@ static void php_phongo_query_free_object(zend_object* object) /* {{{ */
 
 static zend_object* php_phongo_query_create_object(zend_class_entry* class_type) /* {{{ */
 {
-	php_phongo_query_t* intern = NULL;
-
-	intern = PHONGO_ALLOC_OBJECT_T(php_phongo_query_t, class_type);
+	php_phongo_query_t* intern = zend_object_alloc(sizeof(php_phongo_query_t), class_type);
 
 	zend_object_std_init(&intern->std, class_type);
 	object_properties_init(&intern->std, class_type);
@@ -491,12 +491,3 @@ void php_phongo_query_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	php_phongo_handler_query.free_obj       = php_phongo_query_free_object;
 	php_phongo_handler_query.offset         = XtOffsetOf(php_phongo_query_t, std);
 } /* }}} */
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
